"""
Pydantic Models for CMDB - vpn/certificate/setting

Runtime validation models for vpn/certificate/setting configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class SettingCrlVerification(BaseModel):
    """
    Child table model for crl-verification.
    
    CRL verification options.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    expiry: Literal["ignore", "revoke"] | None = Field(default="ignore", description="CRL verification option when CRL is expired (default = ignore).")    
    leaf_crl_absence: Literal["ignore", "revoke"] | None = Field(default="ignore", description="CRL verification option when leaf CRL is absent (default = ignore).")    
    chain_crl_absence: Literal["ignore", "revoke"] | None = Field(default="ignore", description="CRL verification option when CRL of any certificate in chain is absent (default = ignore).")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class SettingSslMinProtoVersionEnum(str, Enum):
    """Allowed values for ssl_min_proto_version field."""
    DEFAULT = "default"
    SSLV3 = "SSLv3"
    TLSV1 = "TLSv1"
    TLSV1_1 = "TLSv1-1"
    TLSV1_2 = "TLSv1-2"
    TLSV1_3 = "TLSv1-3"


# ============================================================================
# Main Model
# ============================================================================

class SettingModel(BaseModel):
    """
    Pydantic model for vpn/certificate/setting configuration.
    
    VPN certificate setting.
    
    Validation Rules:        - ocsp_status: pattern=        - ocsp_option: pattern=        - proxy: max_length=127 pattern=        - proxy_port: min=1 max=65535 pattern=        - proxy_username: max_length=63 pattern=        - proxy_password: max_length=128 pattern=        - source_ip: max_length=63 pattern=        - ocsp_default_server: max_length=35 pattern=        - interface_select_method: pattern=        - interface: max_length=15 pattern=        - vrf_select: min=0 max=511 pattern=        - check_ca_cert: pattern=        - check_ca_chain: pattern=        - subject_match: pattern=        - subject_set: pattern=        - cn_match: pattern=        - cn_allow_multi: pattern=        - crl_verification: pattern=        - strict_ocsp_check: pattern=        - ssl_min_proto_version: pattern=        - cmp_save_extra_certs: pattern=        - cmp_key_usage_checking: pattern=        - cert_expire_warning: min=0 max=100 pattern=        - certname_rsa1024: max_length=35 pattern=        - certname_rsa2048: max_length=35 pattern=        - certname_rsa4096: max_length=35 pattern=        - certname_dsa1024: max_length=35 pattern=        - certname_dsa2048: max_length=35 pattern=        - certname_ecdsa256: max_length=35 pattern=        - certname_ecdsa384: max_length=35 pattern=        - certname_ecdsa521: max_length=35 pattern=        - certname_ed25519: max_length=35 pattern=        - certname_ed448: max_length=35 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    ocsp_status: Literal["enable", "mandatory", "disable"] | None = Field(default="disable", description="Enable/disable receiving certificates using the OCSP.")    
    ocsp_option: Literal["certificate", "server"] | None = Field(default="server", description="Specify whether the OCSP URL is from certificate or configured OCSP server.")    
    proxy: str | None = Field(max_length=127, default=None, description="Proxy server FQDN or IP for OCSP/CA queries during certificate verification.")    
    proxy_port: int | None = Field(ge=1, le=65535, default=8080, description="Proxy server port (1 - 65535, default = 8080).")    
    proxy_username: str | None = Field(max_length=63, default=None, description="Proxy server user name.")    
    proxy_password: Any = Field(max_length=128, default=None, description="Proxy server password.")    
    source_ip: str | None = Field(max_length=63, default=None, description="Source IP address for dynamic AIA and OCSP queries.")    
    ocsp_default_server: str | None = Field(max_length=35, default=None, description="Default OCSP server.")  # datasource: ['vpn.certificate.ocsp-server.name']    
    interface_select_method: Literal["auto", "sdwan", "specify"] | None = Field(default="auto", description="Specify how to select outgoing interface to reach server.")    
    interface: str = Field(max_length=15, description="Specify outgoing interface to reach server.")  # datasource: ['system.interface.name']    
    vrf_select: int | None = Field(ge=0, le=511, default=0, description="VRF ID used for connection to server.")    
    check_ca_cert: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable verification of the user certificate and pass authentication if any CA in the chain is trusted (default = enable).")    
    check_ca_chain: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable verification of the entire certificate chain and pass authentication only if the chain is complete and all of the CAs in the chain are trusted (default = disable).")    
    subject_match: Literal["substring", "value"] | None = Field(default="substring", description="When searching for a matching certificate, control how to do RDN value matching with certificate subject name (default = substring).")    
    subject_set: Literal["subset", "superset"] | None = Field(default="subset", description="When searching for a matching certificate, control how to do RDN set matching with certificate subject name (default = subset).")    
    cn_match: Literal["substring", "value"] | None = Field(default="substring", description="When searching for a matching certificate, control how to do CN value matching with certificate subject name (default = substring).")    
    cn_allow_multi: Literal["disable", "enable"] | None = Field(default="enable", description="When searching for a matching certificate, allow multiple CN fields in certificate subject name (default = enable).")    
    crl_verification: SettingCrlVerification | None = Field(default=None, description="CRL verification options.")    
    strict_ocsp_check: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable strict mode OCSP checking.")    
    ssl_min_proto_version: SettingSslMinProtoVersionEnum | None = Field(default=SettingSslMinProtoVersionEnum.DEFAULT, description="Minimum supported protocol version for SSL/TLS connections (default is to follow system global setting).")    
    cmp_save_extra_certs: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable saving extra certificates in CMP mode (default = disable).")    
    cmp_key_usage_checking: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable server certificate key usage checking in CMP mode (default = enable).")    
    cert_expire_warning: int | None = Field(ge=0, le=100, default=14, description="Number of days before a certificate expires to send a warning. Set to 0 to disable sending of the warning (0 - 100, default = 14).")    
    certname_rsa1024: str = Field(max_length=35, default="Fortinet_SSL_RSA1024", description="1024 bit RSA key certificate for re-signing server certificates for SSL inspection.")  # datasource: ['vpn.certificate.local.name']    
    certname_rsa2048: str = Field(max_length=35, default="Fortinet_SSL_RSA2048", description="2048 bit RSA key certificate for re-signing server certificates for SSL inspection.")  # datasource: ['vpn.certificate.local.name']    
    certname_rsa4096: str = Field(max_length=35, default="Fortinet_SSL_RSA4096", description="4096 bit RSA key certificate for re-signing server certificates for SSL inspection.")  # datasource: ['vpn.certificate.local.name']    
    certname_dsa1024: str = Field(max_length=35, default="Fortinet_SSL_DSA1024", description="1024 bit DSA key certificate for re-signing server certificates for SSL inspection.")  # datasource: ['vpn.certificate.local.name']    
    certname_dsa2048: str = Field(max_length=35, default="Fortinet_SSL_DSA2048", description="2048 bit DSA key certificate for re-signing server certificates for SSL inspection.")  # datasource: ['vpn.certificate.local.name']    
    certname_ecdsa256: str = Field(max_length=35, default="Fortinet_SSL_ECDSA256", description="256 bit ECDSA key certificate for re-signing server certificates for SSL inspection.")  # datasource: ['vpn.certificate.local.name']    
    certname_ecdsa384: str = Field(max_length=35, default="Fortinet_SSL_ECDSA384", description="384 bit ECDSA key certificate for re-signing server certificates for SSL inspection.")  # datasource: ['vpn.certificate.local.name']    
    certname_ecdsa521: str = Field(max_length=35, default="Fortinet_SSL_ECDSA521", description="521 bit ECDSA key certificate for re-signing server certificates for SSL inspection.")  # datasource: ['vpn.certificate.local.name']    
    certname_ed25519: str = Field(max_length=35, default="Fortinet_SSL_ED25519", description="253 bit EdDSA key certificate for re-signing server certificates for SSL inspection.")  # datasource: ['vpn.certificate.local.name']    
    certname_ed448: str = Field(max_length=35, default="Fortinet_SSL_ED448", description="456 bit EdDSA key certificate for re-signing server certificates for SSL inspection.")  # datasource: ['vpn.certificate.local.name']    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('ocsp_default_server')
    @classmethod
    def validate_ocsp_default_server(cls, v: Any) -> Any:
        """
        Validate ocsp_default_server field.
        
        Datasource: ['vpn.certificate.ocsp-server.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('interface')
    @classmethod
    def validate_interface(cls, v: Any) -> Any:
        """
        Validate interface field.
        
        Datasource: ['system.interface.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('certname_rsa1024')
    @classmethod
    def validate_certname_rsa1024(cls, v: Any) -> Any:
        """
        Validate certname_rsa1024 field.
        
        Datasource: ['vpn.certificate.local.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('certname_rsa2048')
    @classmethod
    def validate_certname_rsa2048(cls, v: Any) -> Any:
        """
        Validate certname_rsa2048 field.
        
        Datasource: ['vpn.certificate.local.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('certname_rsa4096')
    @classmethod
    def validate_certname_rsa4096(cls, v: Any) -> Any:
        """
        Validate certname_rsa4096 field.
        
        Datasource: ['vpn.certificate.local.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('certname_dsa1024')
    @classmethod
    def validate_certname_dsa1024(cls, v: Any) -> Any:
        """
        Validate certname_dsa1024 field.
        
        Datasource: ['vpn.certificate.local.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('certname_dsa2048')
    @classmethod
    def validate_certname_dsa2048(cls, v: Any) -> Any:
        """
        Validate certname_dsa2048 field.
        
        Datasource: ['vpn.certificate.local.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('certname_ecdsa256')
    @classmethod
    def validate_certname_ecdsa256(cls, v: Any) -> Any:
        """
        Validate certname_ecdsa256 field.
        
        Datasource: ['vpn.certificate.local.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('certname_ecdsa384')
    @classmethod
    def validate_certname_ecdsa384(cls, v: Any) -> Any:
        """
        Validate certname_ecdsa384 field.
        
        Datasource: ['vpn.certificate.local.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('certname_ecdsa521')
    @classmethod
    def validate_certname_ecdsa521(cls, v: Any) -> Any:
        """
        Validate certname_ecdsa521 field.
        
        Datasource: ['vpn.certificate.local.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('certname_ed25519')
    @classmethod
    def validate_certname_ed25519(cls, v: Any) -> Any:
        """
        Validate certname_ed25519 field.
        
        Datasource: ['vpn.certificate.local.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('certname_ed448')
    @classmethod
    def validate_certname_ed448(cls, v: Any) -> Any:
        """
        Validate certname_ed448 field.
        
        Datasource: ['vpn.certificate.local.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "SettingModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_ocsp_default_server_references(self, client: Any) -> list[str]:
        """
        Validate ocsp_default_server references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - vpn/certificate/ocsp-server        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     ocsp_default_server="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_ocsp_default_server_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.vpn.certificate.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "ocsp_default_server", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.vpn.certificate.ocsp_server.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Ocsp-Default-Server '{value}' not found in "
                "vpn/certificate/ocsp-server"
            )        
        return errors    
    async def validate_interface_references(self, client: Any) -> list[str]:
        """
        Validate interface references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/interface        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     interface="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_interface_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.vpn.certificate.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "interface", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.interface.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Interface '{value}' not found in "
                "system/interface"
            )        
        return errors    
    async def validate_certname_rsa1024_references(self, client: Any) -> list[str]:
        """
        Validate certname_rsa1024 references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - vpn/certificate/local        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     certname_rsa1024="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_certname_rsa1024_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.vpn.certificate.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "certname_rsa1024", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.vpn.certificate.local.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Certname-Rsa1024 '{value}' not found in "
                "vpn/certificate/local"
            )        
        return errors    
    async def validate_certname_rsa2048_references(self, client: Any) -> list[str]:
        """
        Validate certname_rsa2048 references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - vpn/certificate/local        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     certname_rsa2048="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_certname_rsa2048_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.vpn.certificate.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "certname_rsa2048", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.vpn.certificate.local.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Certname-Rsa2048 '{value}' not found in "
                "vpn/certificate/local"
            )        
        return errors    
    async def validate_certname_rsa4096_references(self, client: Any) -> list[str]:
        """
        Validate certname_rsa4096 references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - vpn/certificate/local        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     certname_rsa4096="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_certname_rsa4096_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.vpn.certificate.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "certname_rsa4096", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.vpn.certificate.local.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Certname-Rsa4096 '{value}' not found in "
                "vpn/certificate/local"
            )        
        return errors    
    async def validate_certname_dsa1024_references(self, client: Any) -> list[str]:
        """
        Validate certname_dsa1024 references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - vpn/certificate/local        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     certname_dsa1024="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_certname_dsa1024_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.vpn.certificate.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "certname_dsa1024", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.vpn.certificate.local.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Certname-Dsa1024 '{value}' not found in "
                "vpn/certificate/local"
            )        
        return errors    
    async def validate_certname_dsa2048_references(self, client: Any) -> list[str]:
        """
        Validate certname_dsa2048 references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - vpn/certificate/local        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     certname_dsa2048="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_certname_dsa2048_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.vpn.certificate.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "certname_dsa2048", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.vpn.certificate.local.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Certname-Dsa2048 '{value}' not found in "
                "vpn/certificate/local"
            )        
        return errors    
    async def validate_certname_ecdsa256_references(self, client: Any) -> list[str]:
        """
        Validate certname_ecdsa256 references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - vpn/certificate/local        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     certname_ecdsa256="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_certname_ecdsa256_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.vpn.certificate.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "certname_ecdsa256", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.vpn.certificate.local.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Certname-Ecdsa256 '{value}' not found in "
                "vpn/certificate/local"
            )        
        return errors    
    async def validate_certname_ecdsa384_references(self, client: Any) -> list[str]:
        """
        Validate certname_ecdsa384 references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - vpn/certificate/local        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     certname_ecdsa384="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_certname_ecdsa384_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.vpn.certificate.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "certname_ecdsa384", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.vpn.certificate.local.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Certname-Ecdsa384 '{value}' not found in "
                "vpn/certificate/local"
            )        
        return errors    
    async def validate_certname_ecdsa521_references(self, client: Any) -> list[str]:
        """
        Validate certname_ecdsa521 references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - vpn/certificate/local        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     certname_ecdsa521="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_certname_ecdsa521_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.vpn.certificate.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "certname_ecdsa521", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.vpn.certificate.local.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Certname-Ecdsa521 '{value}' not found in "
                "vpn/certificate/local"
            )        
        return errors    
    async def validate_certname_ed25519_references(self, client: Any) -> list[str]:
        """
        Validate certname_ed25519 references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - vpn/certificate/local        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     certname_ed25519="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_certname_ed25519_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.vpn.certificate.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "certname_ed25519", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.vpn.certificate.local.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Certname-Ed25519 '{value}' not found in "
                "vpn/certificate/local"
            )        
        return errors    
    async def validate_certname_ed448_references(self, client: Any) -> list[str]:
        """
        Validate certname_ed448 references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - vpn/certificate/local        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = SettingModel(
            ...     certname_ed448="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_certname_ed448_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.vpn.certificate.setting.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "certname_ed448", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.vpn.certificate.local.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Certname-Ed448 '{value}' not found in "
                "vpn/certificate/local"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_ocsp_default_server_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_interface_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_certname_rsa1024_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_certname_rsa2048_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_certname_rsa4096_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_certname_dsa1024_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_certname_dsa2048_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_certname_ecdsa256_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_certname_ecdsa384_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_certname_ecdsa521_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_certname_ed25519_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_certname_ed448_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "SettingModel",    "SettingCrlVerification",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:24:01.757198Z
# ============================================================================


"""FortiOS CMDB - Dns category"""

from .latency import Latency

__all__ = [
    "Dns",
    "Latency",
]


class Dns:
    """Dns endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Dns endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.latency = Latency(client)



"""FortiOS CMDB - TrafficHistory category"""

from .enable_app_bandwidth_tracking import EnableAppBandwidthTracking
from .interface import Interface
from .top_applications import TopApplications

__all__ = [
    "EnableAppBandwidthTracking",
    "Interface",
    "TopApplications",
    "TrafficHistory",
]


class TrafficHistory:
    """TrafficHistory endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """TrafficHistory endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.enable_app_bandwidth_tracking = EnableAppBandwidthTracking(client)
        self.interface = Interface(client)
        self.top_applications = TopApplications(client)



"""Type stubs for LOG search endpoints."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient

from hfortix_fortios.models import FortiObjectList


class Search:
    """Search log operations."""
    abort: SearchAbort
    status: SearchStatus

    def __init__(self, client: IHTTPClient) -> None: ...


class SearchAbort:
    """Abort a running log search session."""

    def __init__(self, client: IHTTPClient) -> None: ...

    def get(
        self,
        *,
        rows: int | None = None,
        session_id: int | None = None,
        serial_no: str | None = None,
        is_ha_member: bool | None = None,
        filter: str | None = None,
        extra: str | None = None,
        vdom: str | None = None,
    ) -> FortiObjectList:
        """
        Get abort logs.

        Args:
            rows: Number of rows to return
            session_id: Session ID for paginated retrieval
            serial_no: Retrieve logs from specific device
            is_ha_member: Whether the device is an HA member
            filter: Filter expression
            extra: Extra data flags
            vdom: Virtual domain

        Returns:
            FortiObjectList containing log records
        """
        ...

class SearchStatus:
    """Returns status of log search session, if it is active or not."""

    def __init__(self, client: IHTTPClient) -> None: ...

    def get(
        self,
        *,
        rows: int | None = None,
        session_id: int | None = None,
        serial_no: str | None = None,
        is_ha_member: bool | None = None,
        filter: str | None = None,
        extra: str | None = None,
        vdom: str | None = None,
    ) -> FortiObjectList:
        """
        Get status logs.

        Args:
            rows: Number of rows to return
            session_id: Session ID for paginated retrieval
            serial_no: Retrieve logs from specific device
            is_ha_member: Whether the device is an HA member
            filter: Filter expression
            extra: Extra data flags
            vdom: Virtual domain

        Returns:
            FortiObjectList containing log records
        """
        ...



"""Type stubs for EXTERNAL_RESOURCE category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from .dynamic import Dynamic
    from .entry_list import EntryList
    from .generic_address import GenericAddress
    from .refresh import Refresh
    from .validate_jsonpath import ValidateJsonpath

__all__ = [
    "Dynamic",
    "EntryList",
    "GenericAddress",
    "Refresh",
    "ValidateJsonpath",
    "ExternalResource",
]


class ExternalResource:
    """EXTERNAL_RESOURCE API category.
    
    All endpoints return FortiObject instances with:
    - Attribute access: response.field
    - Dictionary access: response["field"]
    - Convert to dict: response.dict or response.json
    """
    
    dynamic: Dynamic
    entry_list: EntryList
    generic_address: GenericAddress
    refresh: Refresh
    validate_jsonpath: ValidateJsonpath

    def __init__(self, client: IHTTPClient, vdom: str | None = None) -> None:
        """Initialize external_resource category with HTTP client."""
        ...



"""FortiOS CMDB - WirelessController category"""

from . import hotspot20
from .access_control_list import AccessControlList
from .ap_status import ApStatus
from .apcfg_profile import ApcfgProfile
from .arrp_profile import ArrpProfile
from .ble_profile import BleProfile
from .bonjour_profile import BonjourProfile
from .global_ import Global
from .inter_controller import InterController
from .log import Log
from .lw_profile import LwProfile
from .mpsk_profile import MpskProfile
from .nac_profile import NacProfile
from .qos_profile import QosProfile
from .region import Region
from .setting import Setting
from .snmp import Snmp
from .ssid_policy import SsidPolicy
from .syslog_profile import SyslogProfile
from .timers import Timers
from .utm_profile import UtmProfile
from .vap import Vap
from .vap_group import VapGroup
from .wag_profile import WagProfile
from .wids_profile import WidsProfile
from .wtp import Wtp
from .wtp_group import WtpGroup
from .wtp_profile import WtpProfile

__all__ = [
    "AccessControlList",
    "ApStatus",
    "ApcfgProfile",
    "ArrpProfile",
    "BleProfile",
    "BonjourProfile",
    "Global",
    "Hotspot20",
    "InterController",
    "Log",
    "LwProfile",
    "MpskProfile",
    "NacProfile",
    "QosProfile",
    "Region",
    "Setting",
    "Snmp",
    "SsidPolicy",
    "SyslogProfile",
    "Timers",
    "UtmProfile",
    "Vap",
    "VapGroup",
    "WagProfile",
    "WidsProfile",
    "WirelessController",
    "Wtp",
    "WtpGroup",
    "WtpProfile",
]


class WirelessController:
    """WirelessController endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """WirelessController endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.hotspot20 = hotspot20.Hotspot20(client)
        self.access_control_list = AccessControlList(client)
        self.ap_status = ApStatus(client)
        self.apcfg_profile = ApcfgProfile(client)
        self.arrp_profile = ArrpProfile(client)
        self.ble_profile = BleProfile(client)
        self.bonjour_profile = BonjourProfile(client)
        self.global_ = Global(client)
        self.inter_controller = InterController(client)
        self.log = Log(client)
        self.lw_profile = LwProfile(client)
        self.mpsk_profile = MpskProfile(client)
        self.nac_profile = NacProfile(client)
        self.qos_profile = QosProfile(client)
        self.region = Region(client)
        self.setting = Setting(client)
        self.snmp = Snmp(client)
        self.ssid_policy = SsidPolicy(client)
        self.syslog_profile = SyslogProfile(client)
        self.timers = Timers(client)
        self.utm_profile = UtmProfile(client)
        self.vap = Vap(client)
        self.vap_group = VapGroup(client)
        self.wag_profile = WagProfile(client)
        self.wids_profile = WidsProfile(client)
        self.wtp = Wtp(client)
        self.wtp_group = WtpGroup(client)
        self.wtp_profile = WtpProfile(client)



"""FortiOS CMDB - Debug category"""

from .download import Download

__all__ = [
    "Debug",
    "Download",
]


class Debug:
    """Debug endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Debug endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.download = Download(client)



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: extension_controller/dataplan
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class DataplanPayload(TypedDict, total=False):
    """Payload type for Dataplan operations."""
    name: str
    modem_id: Literal["modem1", "modem2", "all"]
    type: Literal["carrier", "slot", "iccid", "generic"]
    slot: Literal["sim1", "sim2"]
    iccid: str
    carrier: str
    apn: str
    auth_type: Literal["none", "pap", "chap"]
    username: str
    password: str
    pdn: Literal["ipv4-only", "ipv6-only", "ipv4-ipv6"]
    signal_threshold: int
    signal_period: int
    capacity: int
    monthly_fee: int
    billing_date: int
    overage: Literal["disable", "enable"]
    preferred_subnet: int
    private_network: Literal["disable", "enable"]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class DataplanResponse(TypedDict, total=False):
    """Response type for Dataplan - use with .dict property for typed dict access."""
    name: str
    modem_id: Literal["modem1", "modem2", "all"]
    type: Literal["carrier", "slot", "iccid", "generic"]
    slot: Literal["sim1", "sim2"]
    iccid: str
    carrier: str
    apn: str
    auth_type: Literal["none", "pap", "chap"]
    username: str
    password: str
    pdn: Literal["ipv4-only", "ipv6-only", "ipv4-ipv6"]
    signal_threshold: int
    signal_period: int
    capacity: int
    monthly_fee: int
    billing_date: int
    overage: Literal["disable", "enable"]
    preferred_subnet: int
    private_network: Literal["disable", "enable"]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class DataplanObject(FortiObject):
    """Typed FortiObject for Dataplan with field access."""
    name: str
    modem_id: Literal["modem1", "modem2", "all"]
    type: Literal["carrier", "slot", "iccid", "generic"]
    slot: Literal["sim1", "sim2"]
    iccid: str
    carrier: str
    apn: str
    auth_type: Literal["none", "pap", "chap"]
    username: str
    password: str
    pdn: Literal["ipv4-only", "ipv6-only", "ipv4-ipv6"]
    signal_threshold: int
    signal_period: int
    capacity: int
    monthly_fee: int
    billing_date: int
    overage: Literal["disable", "enable"]
    preferred_subnet: int
    private_network: Literal["disable", "enable"]


# ================================================================
# Main Endpoint Class
# ================================================================

class Dataplan:
    """
    
    Endpoint: extension_controller/dataplan
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> DataplanObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[DataplanObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: DataplanPayload | None = ...,
        name: str | None = ...,
        modem_id: Literal["modem1", "modem2", "all"] | None = ...,
        type: Literal["carrier", "slot", "iccid", "generic"] | None = ...,
        slot: Literal["sim1", "sim2"] | None = ...,
        iccid: str | None = ...,
        carrier: str | None = ...,
        apn: str | None = ...,
        auth_type: Literal["none", "pap", "chap"] | None = ...,
        username: str | None = ...,
        password: str | None = ...,
        pdn: Literal["ipv4-only", "ipv6-only", "ipv4-ipv6"] | None = ...,
        signal_threshold: int | None = ...,
        signal_period: int | None = ...,
        capacity: int | None = ...,
        monthly_fee: int | None = ...,
        billing_date: int | None = ...,
        overage: Literal["disable", "enable"] | None = ...,
        preferred_subnet: int | None = ...,
        private_network: Literal["disable", "enable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> DataplanObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: DataplanPayload | None = ...,
        name: str | None = ...,
        modem_id: Literal["modem1", "modem2", "all"] | None = ...,
        type: Literal["carrier", "slot", "iccid", "generic"] | None = ...,
        slot: Literal["sim1", "sim2"] | None = ...,
        iccid: str | None = ...,
        carrier: str | None = ...,
        apn: str | None = ...,
        auth_type: Literal["none", "pap", "chap"] | None = ...,
        username: str | None = ...,
        password: str | None = ...,
        pdn: Literal["ipv4-only", "ipv6-only", "ipv4-ipv6"] | None = ...,
        signal_threshold: int | None = ...,
        signal_period: int | None = ...,
        capacity: int | None = ...,
        monthly_fee: int | None = ...,
        billing_date: int | None = ...,
        overage: Literal["disable", "enable"] | None = ...,
        preferred_subnet: int | None = ...,
        private_network: Literal["disable", "enable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> DataplanObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: DataplanPayload | None = ...,
        name: str | None = ...,
        modem_id: Literal["modem1", "modem2", "all"] | None = ...,
        type: Literal["carrier", "slot", "iccid", "generic"] | None = ...,
        slot: Literal["sim1", "sim2"] | None = ...,
        iccid: str | None = ...,
        carrier: str | None = ...,
        apn: str | None = ...,
        auth_type: Literal["none", "pap", "chap"] | None = ...,
        username: str | None = ...,
        password: str | None = ...,
        pdn: Literal["ipv4-only", "ipv6-only", "ipv4-ipv6"] | None = ...,
        signal_threshold: int | None = ...,
        signal_period: int | None = ...,
        capacity: int | None = ...,
        monthly_fee: int | None = ...,
        billing_date: int | None = ...,
        overage: Literal["disable", "enable"] | None = ...,
        preferred_subnet: int | None = ...,
        private_network: Literal["disable", "enable"] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "Dataplan",
    "DataplanPayload",
    "DataplanResponse",
    "DataplanObject",
]


from typing import Any, Literal

# Enum type aliases for validation
VALID_BODY_ACCESS_NETWORK_TYPE: Literal["private-network", "private-network-with-guest-access", "chargeable-public-network", "free-public-network", "personal-device-network", "emergency-services-only-network", "test-or-experimental", "wildcard"]
VALID_BODY_ACCESS_NETWORK_INTERNET: Literal["enable", "disable"]
VALID_BODY_ACCESS_NETWORK_ASRA: Literal["enable", "disable"]
VALID_BODY_ACCESS_NETWORK_ESR: Literal["enable", "disable"]
VALID_BODY_ACCESS_NETWORK_UESA: Literal["enable", "disable"]
VALID_BODY_VENUE_GROUP: Literal["unspecified", "assembly", "business", "educational", "factory", "institutional", "mercantile", "residential", "storage", "utility", "vehicular", "outdoor"]
VALID_BODY_VENUE_TYPE: Literal["unspecified", "arena", "stadium", "passenger-terminal", "amphitheater", "amusement-park", "place-of-worship", "convention-center", "library", "museum", "restaurant", "theater", "bar", "coffee-shop", "zoo-or-aquarium", "emergency-center", "doctor-office", "bank", "fire-station", "police-station", "post-office", "professional-office", "research-facility", "attorney-office", "primary-school", "secondary-school", "university-or-college", "factory", "hospital", "long-term-care-facility", "rehab-center", "group-home", "prison-or-jail", "retail-store", "grocery-market", "auto-service-station", "shopping-mall", "gas-station", "private", "hotel-or-motel", "dormitory", "boarding-house", "automobile", "airplane", "bus", "ferry", "ship-or-boat", "train", "motor-bike", "muni-mesh-network", "city-park", "rest-area", "traffic-control", "bus-stop", "kiosk"]
VALID_BODY_PROXY_ARP: Literal["enable", "disable"]
VALID_BODY_L2TIF: Literal["enable", "disable"]
VALID_BODY_PAME_BI: Literal["disable", "enable"]
VALID_BODY_DGAF: Literal["enable", "disable"]
VALID_BODY_WNM_SLEEP_MODE: Literal["enable", "disable"]
VALID_BODY_BSS_TRANSITION: Literal["enable", "disable"]
VALID_BODY_WBA_OPEN_ROAMING: Literal["disable", "enable"]

# Metadata dictionaries
FIELD_TYPES: dict[str, str]
FIELD_DESCRIPTIONS: dict[str, str]
FIELD_CONSTRAINTS: dict[str, dict[str, Any]]
NESTED_SCHEMAS: dict[str, dict[str, Any]]
FIELDS_WITH_DEFAULTS: dict[str, Any]
DEPRECATED_FIELDS: dict[str, dict[str, str]]
REQUIRED_FIELDS: list[str]

# Helper functions
def get_field_type(field_name: str) -> str | None: ...
def get_field_description(field_name: str) -> str | None: ...
def get_field_default(field_name: str) -> Any: ...
def get_field_constraints(field_name: str) -> dict[str, Any]: ...
def get_nested_schema(field_name: str) -> dict[str, Any] | None: ...
def get_field_metadata(field_name: str) -> dict[str, Any]: ...
def validate_field_value(field_name: str, value: Any) -> bool: ...
def get_all_fields() -> list[str]: ...
def get_required_fields() -> list[str]: ...
def get_schema_info() -> dict[str, Any]: ...


__all__ = [
    "VALID_BODY_ACCESS_NETWORK_TYPE",
    "VALID_BODY_ACCESS_NETWORK_INTERNET",
    "VALID_BODY_ACCESS_NETWORK_ASRA",
    "VALID_BODY_ACCESS_NETWORK_ESR",
    "VALID_BODY_ACCESS_NETWORK_UESA",
    "VALID_BODY_VENUE_GROUP",
    "VALID_BODY_VENUE_TYPE",
    "VALID_BODY_PROXY_ARP",
    "VALID_BODY_L2TIF",
    "VALID_BODY_PAME_BI",
    "VALID_BODY_DGAF",
    "VALID_BODY_WNM_SLEEP_MODE",
    "VALID_BODY_BSS_TRANSITION",
    "VALID_BODY_WBA_OPEN_ROAMING",
    "FIELD_TYPES",
    "FIELD_DESCRIPTIONS",
    "FIELD_CONSTRAINTS",
    "NESTED_SCHEMAS",
    "FIELDS_WITH_DEFAULTS",
    "DEPRECATED_FIELDS",
    "REQUIRED_FIELDS",
    "get_field_type",
    "get_field_description",
    "get_field_default",
    "get_field_constraints",
    "get_nested_schema",
    "get_field_metadata",
    "validate_field_value",
    "get_all_fields",
    "get_required_fields",
    "get_schema_info",
]


"""FortiOS CMDB - Ztna category"""

from .reverse_connector import ReverseConnector
from .traffic_forward_proxy import TrafficForwardProxy
from .web_portal import WebPortal
from .web_portal_bookmark import WebPortalBookmark
from .web_proxy import WebProxy

__all__ = [
    "ReverseConnector",
    "TrafficForwardProxy",
    "WebPortal",
    "WebPortalBookmark",
    "WebProxy",
    "Ztna",
]


class Ztna:
    """Ztna endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Ztna endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.reverse_connector = ReverseConnector(client)
        self.traffic_forward_proxy = TrafficForwardProxy(client)
        self.web_portal = WebPortal(client)
        self.web_portal_bookmark = WebPortalBookmark(client)
        self.web_proxy = WebProxy(client)



"""FortiOS CMDB - Ips category"""

from . import session
from .anomaly import Anomaly
from .hold_signatures import HoldSignatures
from .metadata import Metadata
from .rate_based import RateBased

__all__ = [
    "Anomaly",
    "HoldSignatures",
    "Ips",
    "Metadata",
    "RateBased",
    "Session",
]


class Ips:
    """Ips endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Ips endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.session = session.Session(client)
        self.anomaly = Anomaly(client)
        self.hold_signatures = HoldSignatures(client)
        self.metadata = Metadata(client)
        self.rate_based = RateBased(client)



from typing import Any, Literal

# Enum type aliases for validation
VALID_BODY_OBJECT: Literal["log.fortianalyzer.setting", "log.fortianalyzer.override-setting", "log.fortianalyzer2.setting", "log.fortianalyzer2.override-setting", "log.fortianalyzer3.setting", "log.fortianalyzer3.override-setting", "log.fortianalyzer-cloud.setting", "log.fortianalyzer-cloud.override-setting", "log.syslogd.setting", "log.syslogd.override-setting", "log.syslogd2.setting", "log.syslogd2.override-setting", "log.syslogd3.setting", "log.syslogd3.override-setting", "log.syslogd4.setting", "log.syslogd4.override-setting", "system.gre-tunnel", "system.central-management", "system.csf", "user.radius", "system.interface", "vpn.ipsec.phase1-interface", "vpn.ipsec.phase2-interface", "router.bgp", "router.route-map", "router.prefix-list", "firewall.ippool", "firewall.ippool6", "router.static", "router.static6", "firewall.vip", "firewall.vip6", "system.sdwan", "system.saml", "router.policy", "router.policy6", "log.syslogd.setting", "log.syslogd.override-setting", "firewall.address"]
VALID_BODY_SCOPE: Literal["all", "inclusive", "exclusive"]

# Metadata dictionaries
FIELD_TYPES: dict[str, str]
FIELD_DESCRIPTIONS: dict[str, str]
FIELD_CONSTRAINTS: dict[str, dict[str, Any]]
NESTED_SCHEMAS: dict[str, dict[str, Any]]
FIELDS_WITH_DEFAULTS: dict[str, Any]
DEPRECATED_FIELDS: dict[str, dict[str, str]]
REQUIRED_FIELDS: list[str]

# Helper functions
def get_field_type(field_name: str) -> str | None: ...
def get_field_description(field_name: str) -> str | None: ...
def get_field_default(field_name: str) -> Any: ...
def get_field_constraints(field_name: str) -> dict[str, Any]: ...
def get_nested_schema(field_name: str) -> dict[str, Any] | None: ...
def get_field_metadata(field_name: str) -> dict[str, Any]: ...
def validate_field_value(field_name: str, value: Any) -> bool: ...
def get_all_fields() -> list[str]: ...
def get_required_fields() -> list[str]: ...
def get_schema_info() -> dict[str, Any]: ...


__all__ = [
    "VALID_BODY_OBJECT",
    "VALID_BODY_SCOPE",
    "FIELD_TYPES",
    "FIELD_DESCRIPTIONS",
    "FIELD_CONSTRAINTS",
    "NESTED_SCHEMAS",
    "FIELDS_WITH_DEFAULTS",
    "DEPRECATED_FIELDS",
    "REQUIRED_FIELDS",
    "get_field_type",
    "get_field_description",
    "get_field_default",
    "get_field_constraints",
    "get_nested_schema",
    "get_field_metadata",
    "validate_field_value",
    "get_all_fields",
    "get_required_fields",
    "get_schema_info",
]


"""
Pydantic Models for CMDB - antivirus/settings

Runtime validation models for antivirus/settings configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class SettingsModel(BaseModel):
    """
    Pydantic model for antivirus/settings configuration.
    
    Configure AntiVirus settings.
    
    Validation Rules:        - machine_learning_detection: pattern=        - use_extreme_db: pattern=        - grayware: pattern=        - override_timeout: min=30 max=3600 pattern=        - cache_infected_result: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    machine_learning_detection: Literal["enable", "monitor", "disable"] | None = Field(default="enable", description="Use machine learning based malware detection.")    
    use_extreme_db: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable the use of Extreme AVDB.")    
    grayware: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable grayware detection when an AntiVirus profile is applied to traffic.")    
    override_timeout: int | None = Field(ge=30, le=3600, default=0, description="Override the large file scan timeout value in seconds (30 - 3600). Zero is the default value and is used to disable this command. When disabled, the daemon adjusts the large file scan timeout based on the file size.")    
    cache_infected_result: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable cache of infected scan results (default = enable).")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "SettingsModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "SettingsModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:23:59.587874Z
# ============================================================================


"""Type stubs for MONITOR category."""

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from hfortix_core.http.interface import IHTTPClient
    from . import azure
    from . import casb
    from . import endpoint_control
    from . import extender_controller
    from . import extension_controller
    from . import firewall
    from . import firmware
    from . import fortiguard
    from . import fortiview
    from . import geoip
    from . import ips
    from . import license
    from . import log
    from . import monitor
    from . import network
    from . import registration
    from . import router
    from . import sdwan
    from . import service
    from . import switch_controller
    from . import system
    from . import user
    from . import utm
    from . import videofilter
    from . import virtual_wan
    from . import vpn
    from . import vpn_certificate
    from . import wanopt
    from . import web_ui
    from . import webcache
    from . import webfilter
    from . import webproxy
    from . import wifi

__all__ = [
    "Monitor",
    "MonitorDictMode",
    "MonitorObjectMode",
]

class MonitorDictMode:
    """MONITOR API category for dict response mode.
    
    This class is returned when the client is instantiated with response_mode="dict" (default).
    All endpoints return dict/TypedDict responses by default.
    """
    
    azure: azure.Azure  # No mode classes yet
    casb: casb.Casb  # No mode classes yet
    endpoint_control: endpoint_control.EndpointControl  # No mode classes yet
    extender_controller: extender_controller.ExtenderController  # No mode classes yet
    extension_controller: extension_controller.ExtensionController  # No mode classes yet
    firewall: firewall.Firewall  # No mode classes yet
    firmware: firmware.Firmware  # No mode classes yet
    fortiguard: fortiguard.Fortiguard  # No mode classes yet
    fortiview: fortiview.Fortiview  # No mode classes yet
    geoip: geoip.Geoip  # No mode classes yet
    ips: ips.Ips  # No mode classes yet
    license: license.License  # No mode classes yet
    log: log.Log  # No mode classes yet
    monitor: monitor.Monitor  # No mode classes yet
    network: network.Network  # No mode classes yet
    registration: registration.Registration  # No mode classes yet
    router: router.Router  # No mode classes yet
    sdwan: sdwan.Sdwan  # No mode classes yet
    service: service.Service  # No mode classes yet
    switch_controller: switch_controller.SwitchController  # No mode classes yet
    system: system.System  # No mode classes yet
    user: user.User  # No mode classes yet
    utm: utm.Utm  # No mode classes yet
    videofilter: videofilter.Videofilter  # No mode classes yet
    virtual_wan: virtual_wan.VirtualWan  # No mode classes yet
    vpn: vpn.Vpn  # No mode classes yet
    vpn_certificate: vpn_certificate.VpnCertificate  # No mode classes yet
    wanopt: wanopt.Wanopt  # No mode classes yet
    web_ui: web_ui.WebUi  # No mode classes yet
    webcache: webcache.Webcache  # No mode classes yet
    webfilter: webfilter.Webfilter  # No mode classes yet
    webproxy: webproxy.Webproxy  # No mode classes yet
    wifi: wifi.Wifi  # No mode classes yet

    def __init__(self, client: IHTTPClient) -> None:
        """Initialize MONITOR category with HTTP client."""
        ...


class MonitorObjectMode:
    """MONITOR API category for object response mode.
    
    This class is returned when the client is instantiated with response_mode="object".
    All endpoints return FortiObject responses by default.
    """
    
    azure: azure.Azure  # No mode classes yet
    casb: casb.Casb  # No mode classes yet
    endpoint_control: endpoint_control.EndpointControl  # No mode classes yet
    extender_controller: extender_controller.ExtenderController  # No mode classes yet
    extension_controller: extension_controller.ExtensionController  # No mode classes yet
    firewall: firewall.Firewall  # No mode classes yet
    firmware: firmware.Firmware  # No mode classes yet
    fortiguard: fortiguard.Fortiguard  # No mode classes yet
    fortiview: fortiview.Fortiview  # No mode classes yet
    geoip: geoip.Geoip  # No mode classes yet
    ips: ips.Ips  # No mode classes yet
    license: license.License  # No mode classes yet
    log: log.Log  # No mode classes yet
    monitor: monitor.Monitor  # No mode classes yet
    network: network.Network  # No mode classes yet
    registration: registration.Registration  # No mode classes yet
    router: router.Router  # No mode classes yet
    sdwan: sdwan.Sdwan  # No mode classes yet
    service: service.Service  # No mode classes yet
    switch_controller: switch_controller.SwitchController  # No mode classes yet
    system: system.System  # No mode classes yet
    user: user.User  # No mode classes yet
    utm: utm.Utm  # No mode classes yet
    videofilter: videofilter.Videofilter  # No mode classes yet
    virtual_wan: virtual_wan.VirtualWan  # No mode classes yet
    vpn: vpn.Vpn  # No mode classes yet
    vpn_certificate: vpn_certificate.VpnCertificate  # No mode classes yet
    wanopt: wanopt.Wanopt  # No mode classes yet
    web_ui: web_ui.WebUi  # No mode classes yet
    webcache: webcache.Webcache  # No mode classes yet
    webfilter: webfilter.Webfilter  # No mode classes yet
    webproxy: webproxy.Webproxy  # No mode classes yet
    wifi: wifi.Wifi  # No mode classes yet

    def __init__(self, client: IHTTPClient) -> None:
        """Initialize MONITOR category with HTTP client."""
        ...


# Base class for backwards compatibility
class Monitor:
    """MONITOR API category."""
    
    azure: azure.Azure
    casb: casb.Casb
    endpoint_control: endpoint_control.EndpointControl
    extender_controller: extender_controller.ExtenderController
    extension_controller: extension_controller.ExtensionController
    firewall: firewall.Firewall
    firmware: firmware.Firmware
    fortiguard: fortiguard.Fortiguard
    fortiview: fortiview.Fortiview
    geoip: geoip.Geoip
    ips: ips.Ips
    license: license.License
    log: log.Log
    monitor: monitor.Monitor
    network: network.Network
    registration: registration.Registration
    router: router.Router
    sdwan: sdwan.Sdwan
    service: service.Service
    switch_controller: switch_controller.SwitchController
    system: system.System
    user: user.User
    utm: utm.Utm
    videofilter: videofilter.Videofilter
    virtual_wan: virtual_wan.VirtualWan
    vpn: vpn.Vpn
    vpn_certificate: vpn_certificate.VpnCertificate
    wanopt: wanopt.Wanopt
    web_ui: web_ui.WebUi
    webcache: webcache.Webcache
    webfilter: webfilter.Webfilter
    webproxy: webproxy.Webproxy
    wifi: wifi.Wifi

    def __init__(self, client: IHTTPClient) -> None:
        """Initialize MONITOR category with HTTP client."""
        ...


"""FortiOS CMDB - ComLog category"""

from .download import Download
from .dump import Dump
from .update import Update

__all__ = [
    "ComLog",
    "Download",
    "Dump",
    "Update",
]


class ComLog:
    """ComLog endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """ComLog endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.download = Download(client)
        self.dump = Dump(client)
        self.update = Update(client)



"""FortiOS CMDB - Proxy category"""

from .sessions import Sessions

__all__ = [
    "Proxy",
    "Sessions",
]


class Proxy:
    """Proxy endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Proxy endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.sessions = Sessions(client)



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: system/ike
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class IkeDhgroup1Dict(TypedDict, total=False):
    """Nested object type for dh-group-1 field."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup2Dict(TypedDict, total=False):
    """Nested object type for dh-group-2 field."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup5Dict(TypedDict, total=False):
    """Nested object type for dh-group-5 field."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup14Dict(TypedDict, total=False):
    """Nested object type for dh-group-14 field."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup15Dict(TypedDict, total=False):
    """Nested object type for dh-group-15 field."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup16Dict(TypedDict, total=False):
    """Nested object type for dh-group-16 field."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup17Dict(TypedDict, total=False):
    """Nested object type for dh-group-17 field."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup18Dict(TypedDict, total=False):
    """Nested object type for dh-group-18 field."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup19Dict(TypedDict, total=False):
    """Nested object type for dh-group-19 field."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup20Dict(TypedDict, total=False):
    """Nested object type for dh-group-20 field."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup21Dict(TypedDict, total=False):
    """Nested object type for dh-group-21 field."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup27Dict(TypedDict, total=False):
    """Nested object type for dh-group-27 field."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup28Dict(TypedDict, total=False):
    """Nested object type for dh-group-28 field."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup29Dict(TypedDict, total=False):
    """Nested object type for dh-group-29 field."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup30Dict(TypedDict, total=False):
    """Nested object type for dh-group-30 field."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup31Dict(TypedDict, total=False):
    """Nested object type for dh-group-31 field."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup32Dict(TypedDict, total=False):
    """Nested object type for dh-group-32 field."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkePayload(TypedDict, total=False):
    """Payload type for Ike operations."""
    embryonic_limit: int
    dh_multiprocess: Literal["enable", "disable"]
    dh_worker_count: int
    dh_mode: Literal["software", "hardware"]
    dh_keypair_cache: Literal["enable", "disable"]
    dh_keypair_count: int
    dh_keypair_throttle: Literal["enable", "disable"]
    dh_group_1: IkeDhgroup1Dict
    dh_group_2: IkeDhgroup2Dict
    dh_group_5: IkeDhgroup5Dict
    dh_group_14: IkeDhgroup14Dict
    dh_group_15: IkeDhgroup15Dict
    dh_group_16: IkeDhgroup16Dict
    dh_group_17: IkeDhgroup17Dict
    dh_group_18: IkeDhgroup18Dict
    dh_group_19: IkeDhgroup19Dict
    dh_group_20: IkeDhgroup20Dict
    dh_group_21: IkeDhgroup21Dict
    dh_group_27: IkeDhgroup27Dict
    dh_group_28: IkeDhgroup28Dict
    dh_group_29: IkeDhgroup29Dict
    dh_group_30: IkeDhgroup30Dict
    dh_group_31: IkeDhgroup31Dict
    dh_group_32: IkeDhgroup32Dict


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class IkeResponse(TypedDict, total=False):
    """Response type for Ike - use with .dict property for typed dict access."""
    embryonic_limit: int
    dh_multiprocess: Literal["enable", "disable"]
    dh_worker_count: int
    dh_mode: Literal["software", "hardware"]
    dh_keypair_cache: Literal["enable", "disable"]
    dh_keypair_count: int
    dh_keypair_throttle: Literal["enable", "disable"]
    dh_group_1: IkeDhgroup1Dict
    dh_group_2: IkeDhgroup2Dict
    dh_group_5: IkeDhgroup5Dict
    dh_group_14: IkeDhgroup14Dict
    dh_group_15: IkeDhgroup15Dict
    dh_group_16: IkeDhgroup16Dict
    dh_group_17: IkeDhgroup17Dict
    dh_group_18: IkeDhgroup18Dict
    dh_group_19: IkeDhgroup19Dict
    dh_group_20: IkeDhgroup20Dict
    dh_group_21: IkeDhgroup21Dict
    dh_group_27: IkeDhgroup27Dict
    dh_group_28: IkeDhgroup28Dict
    dh_group_29: IkeDhgroup29Dict
    dh_group_30: IkeDhgroup30Dict
    dh_group_31: IkeDhgroup31Dict
    dh_group_32: IkeDhgroup32Dict


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class IkeDhgroup1Object(FortiObject):
    """Nested object for dh-group-1 field with attribute access."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup2Object(FortiObject):
    """Nested object for dh-group-2 field with attribute access."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup5Object(FortiObject):
    """Nested object for dh-group-5 field with attribute access."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup14Object(FortiObject):
    """Nested object for dh-group-14 field with attribute access."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup15Object(FortiObject):
    """Nested object for dh-group-15 field with attribute access."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup16Object(FortiObject):
    """Nested object for dh-group-16 field with attribute access."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup17Object(FortiObject):
    """Nested object for dh-group-17 field with attribute access."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup18Object(FortiObject):
    """Nested object for dh-group-18 field with attribute access."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup19Object(FortiObject):
    """Nested object for dh-group-19 field with attribute access."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup20Object(FortiObject):
    """Nested object for dh-group-20 field with attribute access."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup21Object(FortiObject):
    """Nested object for dh-group-21 field with attribute access."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup27Object(FortiObject):
    """Nested object for dh-group-27 field with attribute access."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup28Object(FortiObject):
    """Nested object for dh-group-28 field with attribute access."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup29Object(FortiObject):
    """Nested object for dh-group-29 field with attribute access."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup30Object(FortiObject):
    """Nested object for dh-group-30 field with attribute access."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup31Object(FortiObject):
    """Nested object for dh-group-31 field with attribute access."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeDhgroup32Object(FortiObject):
    """Nested object for dh-group-32 field with attribute access."""
    mode: Literal["software", "hardware", "global"]
    keypair_cache: Literal["global", "custom"]
    keypair_count: int


class IkeObject(FortiObject):
    """Typed FortiObject for Ike with field access."""
    embryonic_limit: int
    dh_multiprocess: Literal["enable", "disable"]
    dh_worker_count: int
    dh_mode: Literal["software", "hardware"]
    dh_keypair_cache: Literal["enable", "disable"]
    dh_keypair_count: int
    dh_keypair_throttle: Literal["enable", "disable"]
    dh_group_1: IkeDhgroup1Object
    dh_group_2: IkeDhgroup2Object
    dh_group_5: IkeDhgroup5Object
    dh_group_14: IkeDhgroup14Object
    dh_group_15: IkeDhgroup15Object
    dh_group_16: IkeDhgroup16Object
    dh_group_17: IkeDhgroup17Object
    dh_group_18: IkeDhgroup18Object
    dh_group_19: IkeDhgroup19Object
    dh_group_20: IkeDhgroup20Object
    dh_group_21: IkeDhgroup21Object
    dh_group_27: IkeDhgroup27Object
    dh_group_28: IkeDhgroup28Object
    dh_group_29: IkeDhgroup29Object
    dh_group_30: IkeDhgroup30Object
    dh_group_31: IkeDhgroup31Object
    dh_group_32: IkeDhgroup32Object


# ================================================================
# Main Endpoint Class
# ================================================================

class Ike:
    """
    
    Endpoint: system/ike
    Category: cmdb
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # Singleton endpoint (no mkey)
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> IkeObject: ...
    
    def get_schema(
        self,
        format: str = ...,
    ) -> FortiObject: ...


    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: IkePayload | None = ...,
        embryonic_limit: int | None = ...,
        dh_multiprocess: Literal["enable", "disable"] | None = ...,
        dh_worker_count: int | None = ...,
        dh_mode: Literal["software", "hardware"] | None = ...,
        dh_keypair_cache: Literal["enable", "disable"] | None = ...,
        dh_keypair_count: int | None = ...,
        dh_keypair_throttle: Literal["enable", "disable"] | None = ...,
        dh_group_1: IkeDhgroup1Dict | None = ...,
        dh_group_2: IkeDhgroup2Dict | None = ...,
        dh_group_5: IkeDhgroup5Dict | None = ...,
        dh_group_14: IkeDhgroup14Dict | None = ...,
        dh_group_15: IkeDhgroup15Dict | None = ...,
        dh_group_16: IkeDhgroup16Dict | None = ...,
        dh_group_17: IkeDhgroup17Dict | None = ...,
        dh_group_18: IkeDhgroup18Dict | None = ...,
        dh_group_19: IkeDhgroup19Dict | None = ...,
        dh_group_20: IkeDhgroup20Dict | None = ...,
        dh_group_21: IkeDhgroup21Dict | None = ...,
        dh_group_27: IkeDhgroup27Dict | None = ...,
        dh_group_28: IkeDhgroup28Dict | None = ...,
        dh_group_29: IkeDhgroup29Dict | None = ...,
        dh_group_30: IkeDhgroup30Dict | None = ...,
        dh_group_31: IkeDhgroup31Dict | None = ...,
        dh_group_32: IkeDhgroup32Dict | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> IkeObject: ...


    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: IkePayload | None = ...,
        embryonic_limit: int | None = ...,
        dh_multiprocess: Literal["enable", "disable"] | None = ...,
        dh_worker_count: int | None = ...,
        dh_mode: Literal["software", "hardware"] | None = ...,
        dh_keypair_cache: Literal["enable", "disable"] | None = ...,
        dh_keypair_count: int | None = ...,
        dh_keypair_throttle: Literal["enable", "disable"] | None = ...,
        dh_group_1: IkeDhgroup1Dict | None = ...,
        dh_group_2: IkeDhgroup2Dict | None = ...,
        dh_group_5: IkeDhgroup5Dict | None = ...,
        dh_group_14: IkeDhgroup14Dict | None = ...,
        dh_group_15: IkeDhgroup15Dict | None = ...,
        dh_group_16: IkeDhgroup16Dict | None = ...,
        dh_group_17: IkeDhgroup17Dict | None = ...,
        dh_group_18: IkeDhgroup18Dict | None = ...,
        dh_group_19: IkeDhgroup19Dict | None = ...,
        dh_group_20: IkeDhgroup20Dict | None = ...,
        dh_group_21: IkeDhgroup21Dict | None = ...,
        dh_group_27: IkeDhgroup27Dict | None = ...,
        dh_group_28: IkeDhgroup28Dict | None = ...,
        dh_group_29: IkeDhgroup29Dict | None = ...,
        dh_group_30: IkeDhgroup30Dict | None = ...,
        dh_group_31: IkeDhgroup31Dict | None = ...,
        dh_group_32: IkeDhgroup32Dict | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject[Any]: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject[Any]: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject[Any]: ...
    
    @staticmethod
    def schema() -> FortiObject[Any]: ...


__all__ = [
    "Ike",
    "IkePayload",
    "IkeResponse",
    "IkeObject",
]


"""
Pydantic Models for CMDB - switch_controller/initial_config/vlans

Runtime validation models for switch_controller/initial_config/vlans configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class VlansModel(BaseModel):
    """
    Pydantic model for switch_controller/initial_config/vlans configuration.
    
    Configure initial template for auto-generated VLAN interfaces.
    
    Validation Rules:        - optional_vlans: pattern=        - default_vlan: max_length=63 pattern=        - quarantine: max_length=63 pattern=        - rspan: max_length=63 pattern=        - voice: max_length=63 pattern=        - video: max_length=63 pattern=        - nac: max_length=63 pattern=        - nac_segment: max_length=63 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    optional_vlans: Literal["enable", "disable"] | None = Field(default="enable", description="Auto-generate pre-configured VLANs upon switch discovery.")    
    default_vlan: str | None = Field(max_length=63, default="_default", description="Default VLAN (native) assigned to all switch ports upon discovery.")  # datasource: ['switch-controller.initial-config.template.name']    
    quarantine: str | None = Field(max_length=63, default="quarantine", description="VLAN for quarantined traffic.")  # datasource: ['switch-controller.initial-config.template.name']    
    rspan: str | None = Field(max_length=63, default="rspan", description="VLAN for RSPAN/ERSPAN mirrored traffic.")  # datasource: ['switch-controller.initial-config.template.name']    
    voice: str | None = Field(max_length=63, default="voice", description="VLAN dedicated for voice devices.")  # datasource: ['switch-controller.initial-config.template.name']    
    video: str | None = Field(max_length=63, default="video", description="VLAN dedicated for video devices.")  # datasource: ['switch-controller.initial-config.template.name']    
    nac: str | None = Field(max_length=63, default="onboarding", description="VLAN for NAC onboarding devices.")  # datasource: ['switch-controller.initial-config.template.name']    
    nac_segment: str | None = Field(max_length=63, default="nac_segment", description="VLAN for NAC segment primary interface.")  # datasource: ['switch-controller.initial-config.template.name']    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('default_vlan')
    @classmethod
    def validate_default_vlan(cls, v: Any) -> Any:
        """
        Validate default_vlan field.
        
        Datasource: ['switch-controller.initial-config.template.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('quarantine')
    @classmethod
    def validate_quarantine(cls, v: Any) -> Any:
        """
        Validate quarantine field.
        
        Datasource: ['switch-controller.initial-config.template.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('rspan')
    @classmethod
    def validate_rspan(cls, v: Any) -> Any:
        """
        Validate rspan field.
        
        Datasource: ['switch-controller.initial-config.template.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('voice')
    @classmethod
    def validate_voice(cls, v: Any) -> Any:
        """
        Validate voice field.
        
        Datasource: ['switch-controller.initial-config.template.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('video')
    @classmethod
    def validate_video(cls, v: Any) -> Any:
        """
        Validate video field.
        
        Datasource: ['switch-controller.initial-config.template.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('nac')
    @classmethod
    def validate_nac(cls, v: Any) -> Any:
        """
        Validate nac field.
        
        Datasource: ['switch-controller.initial-config.template.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('nac_segment')
    @classmethod
    def validate_nac_segment(cls, v: Any) -> Any:
        """
        Validate nac_segment field.
        
        Datasource: ['switch-controller.initial-config.template.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "VlansModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_default_vlan_references(self, client: Any) -> list[str]:
        """
        Validate default_vlan references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - switch-controller/initial-config/template        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VlansModel(
            ...     default_vlan="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_default_vlan_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.switch_controller.initial_config.vlans.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "default_vlan", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.switch_controller.initial_config.template.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Default-Vlan '{value}' not found in "
                "switch-controller/initial-config/template"
            )        
        return errors    
    async def validate_quarantine_references(self, client: Any) -> list[str]:
        """
        Validate quarantine references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - switch-controller/initial-config/template        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VlansModel(
            ...     quarantine="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_quarantine_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.switch_controller.initial_config.vlans.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "quarantine", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.switch_controller.initial_config.template.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Quarantine '{value}' not found in "
                "switch-controller/initial-config/template"
            )        
        return errors    
    async def validate_rspan_references(self, client: Any) -> list[str]:
        """
        Validate rspan references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - switch-controller/initial-config/template        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VlansModel(
            ...     rspan="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_rspan_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.switch_controller.initial_config.vlans.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "rspan", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.switch_controller.initial_config.template.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Rspan '{value}' not found in "
                "switch-controller/initial-config/template"
            )        
        return errors    
    async def validate_voice_references(self, client: Any) -> list[str]:
        """
        Validate voice references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - switch-controller/initial-config/template        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VlansModel(
            ...     voice="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_voice_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.switch_controller.initial_config.vlans.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "voice", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.switch_controller.initial_config.template.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Voice '{value}' not found in "
                "switch-controller/initial-config/template"
            )        
        return errors    
    async def validate_video_references(self, client: Any) -> list[str]:
        """
        Validate video references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - switch-controller/initial-config/template        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VlansModel(
            ...     video="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_video_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.switch_controller.initial_config.vlans.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "video", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.switch_controller.initial_config.template.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Video '{value}' not found in "
                "switch-controller/initial-config/template"
            )        
        return errors    
    async def validate_nac_references(self, client: Any) -> list[str]:
        """
        Validate nac references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - switch-controller/initial-config/template        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VlansModel(
            ...     nac="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_nac_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.switch_controller.initial_config.vlans.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "nac", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.switch_controller.initial_config.template.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Nac '{value}' not found in "
                "switch-controller/initial-config/template"
            )        
        return errors    
    async def validate_nac_segment_references(self, client: Any) -> list[str]:
        """
        Validate nac_segment references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - switch-controller/initial-config/template        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = VlansModel(
            ...     nac_segment="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_nac_segment_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.switch_controller.initial_config.vlans.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "nac_segment", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.switch_controller.initial_config.template.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Nac-Segment '{value}' not found in "
                "switch-controller/initial-config/template"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_default_vlan_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_quarantine_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_rspan_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_voice_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_video_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_nac_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_nac_segment_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "VlansModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:24:01.291060Z
# ============================================================================


"""Validation helpers for system/saml - Auto-generated"""

from typing import Any, TypedDict, Literal
from typing_extensions import NotRequired

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
    "default-profile",  # Default profile for new SSO admin.
    "entity-id",  # SP entity ID.
    "idp-cert",  # IDP certificate name.
    "server-address",  # Server address.
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "status": "disable",
    "role": "service-provider",
    "default-login-page": "normal",
    "default-profile": "",
    "cert": "",
    "binding-protocol": "redirect",
    "portal-url": "",
    "entity-id": "",
    "single-sign-on-url": "",
    "single-logout-url": "",
    "idp-entity-id": "",
    "idp-single-sign-on-url": "",
    "idp-single-logout-url": "",
    "idp-cert": "",
    "server-address": "",
    "require-signed-resp-and-asrt": "disable",
    "tolerance": 5,
    "life": 30,
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "status": "option",  # Enable/disable SAML authentication (default = disable).
    "role": "option",  # SAML role.
    "default-login-page": "option",  # Choose default login page.
    "default-profile": "string",  # Default profile for new SSO admin.
    "cert": "string",  # Certificate to sign SAML messages.
    "binding-protocol": "option",  # IdP Binding protocol.
    "portal-url": "string",  # SP portal URL.
    "entity-id": "string",  # SP entity ID.
    "single-sign-on-url": "string",  # SP single sign-on URL.
    "single-logout-url": "string",  # SP single logout URL.
    "idp-entity-id": "string",  # IDP entity ID.
    "idp-single-sign-on-url": "string",  # IDP single sign-on URL.
    "idp-single-logout-url": "string",  # IDP single logout URL.
    "idp-cert": "string",  # IDP certificate name.
    "server-address": "string",  # Server address.
    "require-signed-resp-and-asrt": "option",  # Require both response and assertion from IDP to be signed wh
    "tolerance": "integer",  # Tolerance to the range of time when the assertion is valid (
    "life": "integer",  # Length of the range of time when the assertion is valid (in 
    "service-providers": "string",  # Authorized service providers.
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "status": "Enable/disable SAML authentication (default = disable).",
    "role": "SAML role.",
    "default-login-page": "Choose default login page.",
    "default-profile": "Default profile for new SSO admin.",
    "cert": "Certificate to sign SAML messages.",
    "binding-protocol": "IdP Binding protocol.",
    "portal-url": "SP portal URL.",
    "entity-id": "SP entity ID.",
    "single-sign-on-url": "SP single sign-on URL.",
    "single-logout-url": "SP single logout URL.",
    "idp-entity-id": "IDP entity ID.",
    "idp-single-sign-on-url": "IDP single sign-on URL.",
    "idp-single-logout-url": "IDP single logout URL.",
    "idp-cert": "IDP certificate name.",
    "server-address": "Server address.",
    "require-signed-resp-and-asrt": "Require both response and assertion from IDP to be signed when FGT acts as SP (default = disable).",
    "tolerance": "Tolerance to the range of time when the assertion is valid (in minutes).",
    "life": "Length of the range of time when the assertion is valid (in minutes).",
    "service-providers": "Authorized service providers.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "default-profile": {"type": "string", "max_length": 35},
    "cert": {"type": "string", "max_length": 35},
    "portal-url": {"type": "string", "max_length": 255},
    "entity-id": {"type": "string", "max_length": 255},
    "single-sign-on-url": {"type": "string", "max_length": 255},
    "single-logout-url": {"type": "string", "max_length": 255},
    "idp-entity-id": {"type": "string", "max_length": 255},
    "idp-single-sign-on-url": {"type": "string", "max_length": 255},
    "idp-single-logout-url": {"type": "string", "max_length": 255},
    "idp-cert": {"type": "string", "max_length": 35},
    "server-address": {"type": "string", "max_length": 63},
    "tolerance": {"type": "integer", "min": 0, "max": 4294967295},
    "life": {"type": "integer", "min": 0, "max": 4294967295},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
    "service-providers": {
        "name": {
            "type": "string",
            "help": "Name.",
            "required": True,
            "default": "",
            "max_length": 35,
        },
        "prefix": {
            "type": "string",
            "help": "Prefix.",
            "required": True,
            "default": "",
            "max_length": 35,
        },
        "sp-binding-protocol": {
            "type": "option",
            "help": "SP binding protocol.",
            "default": "post",
            "options": ["post", "redirect"],
        },
        "sp-cert": {
            "type": "string",
            "help": "SP certificate name.",
            "default": "",
            "max_length": 35,
        },
        "sp-entity-id": {
            "type": "string",
            "help": "SP entity ID.",
            "required": True,
            "default": "",
            "max_length": 255,
        },
        "sp-single-sign-on-url": {
            "type": "string",
            "help": "SP single sign-on URL.",
            "required": True,
            "default": "",
            "max_length": 255,
        },
        "sp-single-logout-url": {
            "type": "string",
            "help": "SP single logout URL.",
            "default": "",
            "max_length": 255,
        },
        "sp-portal-url": {
            "type": "string",
            "help": "SP portal URL.",
            "default": "",
            "max_length": 255,
        },
        "idp-entity-id": {
            "type": "string",
            "help": "IDP entity ID.",
            "default": "",
            "max_length": 255,
        },
        "idp-single-sign-on-url": {
            "type": "string",
            "help": "IDP single sign-on URL.",
            "default": "",
            "max_length": 255,
        },
        "idp-single-logout-url": {
            "type": "string",
            "help": "IDP single logout URL.",
            "default": "",
            "max_length": 255,
        },
        "assertion-attributes": {
            "type": "string",
            "help": "Customized SAML attributes to send along with assertion.",
        },
    },
}


# Valid enum values from API documentation
VALID_BODY_STATUS = [
    "enable",
    "disable",
]
VALID_BODY_ROLE = [
    "identity-provider",
    "service-provider",
]
VALID_BODY_DEFAULT_LOGIN_PAGE = [
    "normal",
    "sso",
]
VALID_BODY_BINDING_PROTOCOL = [
    "post",
    "redirect",
]
VALID_BODY_REQUIRE_SIGNED_RESP_AND_ASRT = [
    "enable",
    "disable",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_system_saml_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for system/saml."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_system_saml_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new system/saml object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "status" in payload:
        is_valid, error = _validate_enum_field(
            "status",
            payload["status"],
            VALID_BODY_STATUS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "role" in payload:
        is_valid, error = _validate_enum_field(
            "role",
            payload["role"],
            VALID_BODY_ROLE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "default-login-page" in payload:
        is_valid, error = _validate_enum_field(
            "default-login-page",
            payload["default-login-page"],
            VALID_BODY_DEFAULT_LOGIN_PAGE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "binding-protocol" in payload:
        is_valid, error = _validate_enum_field(
            "binding-protocol",
            payload["binding-protocol"],
            VALID_BODY_BINDING_PROTOCOL,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "require-signed-resp-and-asrt" in payload:
        is_valid, error = _validate_enum_field(
            "require-signed-resp-and-asrt",
            payload["require-signed-resp-and-asrt"],
            VALID_BODY_REQUIRE_SIGNED_RESP_AND_ASRT,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_system_saml_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update system/saml."""
    # Validate enum values using central function
    if "status" in payload:
        is_valid, error = _validate_enum_field(
            "status",
            payload["status"],
            VALID_BODY_STATUS,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "role" in payload:
        is_valid, error = _validate_enum_field(
            "role",
            payload["role"],
            VALID_BODY_ROLE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "default-login-page" in payload:
        is_valid, error = _validate_enum_field(
            "default-login-page",
            payload["default-login-page"],
            VALID_BODY_DEFAULT_LOGIN_PAGE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "binding-protocol" in payload:
        is_valid, error = _validate_enum_field(
            "binding-protocol",
            payload["binding-protocol"],
            VALID_BODY_BINDING_PROTOCOL,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "require-signed-resp-and-asrt" in payload:
        is_valid, error = _validate_enum_field(
            "require-signed-resp-and-asrt",
            payload["require-signed-resp-and-asrt"],
            VALID_BODY_REQUIRE_SIGNED_RESP_AND_ASRT,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# Bound to this endpoint's data using functools.partial (saves ~7KB per file)
# ============================================================================

from functools import partial
from hfortix_fortios._helpers.metadata import (
    get_field_description,
    get_field_type,
    get_field_constraints,
    get_field_default,
    get_field_options,
    get_nested_schema,
    get_all_fields,
    get_field_metadata,
    validate_field_value,
)

# Bind module-specific data to central functions using partial application
get_field_description = partial(get_field_description, FIELD_DESCRIPTIONS)
get_field_type = partial(get_field_type, FIELD_TYPES)
get_field_constraints = partial(get_field_constraints, FIELD_CONSTRAINTS)
get_field_default = partial(get_field_default, FIELDS_WITH_DEFAULTS)
get_field_options = partial(get_field_options, globals())
get_nested_schema = partial(get_nested_schema, NESTED_SCHEMAS)
get_all_fields = partial(get_all_fields, FIELD_TYPES)
get_field_metadata = partial(get_field_metadata, FIELD_TYPES, FIELD_DESCRIPTIONS, 
                             FIELD_CONSTRAINTS, FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS,
                             NESTED_SCHEMAS, globals())
validate_field_value = partial(validate_field_value, FIELD_TYPES, FIELD_DESCRIPTIONS,
                               FIELD_CONSTRAINTS, globals())


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "system/saml",
    "category": "cmdb",
    "api_path": "system/saml",
    "help": "Global settings for SAML authentication.",
    "total_fields": 19,
    "required_fields_count": 4,
    "fields_with_defaults_count": 18,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



"""FortiOS CMDB - Qos category"""

from .dot1p_map import Dot1pMap
from .ip_dscp_map import IpDscpMap
from .qos_policy import QosPolicy
from .queue_policy import QueuePolicy

__all__ = [
    "Dot1pMap",
    "IpDscpMap",
    "Qos",
    "QosPolicy",
    "QueuePolicy",
]


class Qos:
    """Qos endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Qos endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.dot1p_map = Dot1pMap(client)
        self.ip_dscp_map = IpDscpMap(client)
        self.qos_policy = QosPolicy(client)
        self.queue_policy = QueuePolicy(client)



"""
Pydantic Models for CMDB - emailfilter/profile

Runtime validation models for emailfilter/profile configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field, field_validator
from typing import Any, Literal, Optional
from enum import Enum

# ============================================================================
# Enum Definitions for Child Table Fields (for fields with 4+ allowed values)
# ============================================================================

# ============================================================================
# Child Table Models (sorted deepest-first so nested models are defined before their parents)
# ============================================================================

class ProfileYahooMail(BaseModel):
    """
    Child table model for yahoo-mail.
    
    Yahoo! Mail.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    log_all: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable logging of all email traffic.")
class ProfileSmtp(BaseModel):
    """
    Child table model for smtp.
    
    SMTP.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    log_all: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable logging of all email traffic.")    
    action: Literal["pass", "tag", "discard"] | None = Field(default="discard", description="Action for spam email.")    
    tag_type: list[Literal["subject", "header", "spaminfo"]] = Field(default_factory=list, description="Tag subject or header for spam email.")    
    tag_msg: str | None = Field(max_length=63, default="Spam", description="Subject text or header added to spam email.")    
    hdrip: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable SMTP email header IP checks for spamfsip, spamrbl, and spambal filters.")    
    local_override: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable local filter to override SMTP remote check result.")
class ProfilePop3(BaseModel):
    """
    Child table model for pop3.
    
    POP3.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    log_all: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable logging of all email traffic.")    
    action: Literal["pass", "tag"] | None = Field(default="tag", description="Action for spam email.")    
    tag_type: list[Literal["subject", "header", "spaminfo"]] = Field(default_factory=list, description="Tag subject or header for spam email.")    
    tag_msg: str | None = Field(max_length=63, default="Spam", description="Subject text or header added to spam email.")
class ProfileOtherWebmails(BaseModel):
    """
    Child table model for other-webmails.
    
    Other supported webmails.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    log_all: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable logging of all email traffic.")
class ProfileMsnHotmail(BaseModel):
    """
    Child table model for msn-hotmail.
    
    MSN Hotmail.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    log_all: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable logging of all email traffic.")
class ProfileMapi(BaseModel):
    """
    Child table model for mapi.
    
    MAPI.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    log_all: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable logging of all email traffic.")    
    action: Literal["pass", "discard"] | None = Field(default="pass", description="Action for spam email.")
class ProfileImap(BaseModel):
    """
    Child table model for imap.
    
    IMAP.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    log_all: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable logging of all email traffic.")    
    action: Literal["pass", "tag"] | None = Field(default="tag", description="Action for spam email.")    
    tag_type: list[Literal["subject", "header", "spaminfo"]] = Field(default_factory=list, description="Tag subject or header for spam email.")    
    tag_msg: str | None = Field(max_length=63, default="Spam", description="Subject text or header added to spam email.")
class ProfileGmail(BaseModel):
    """
    Child table model for gmail.
    
    Gmail.
    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        use_enum_values = True  # Use enum values instead of names
    
    log_all: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable logging of all email traffic.")
# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================

class ProfileOptionsEnum(str, Enum):
    """Allowed values for options field."""
    BANNEDWORD = "bannedword"
    SPAMBAL = "spambal"
    SPAMFSIP = "spamfsip"
    SPAMFSSUBMIT = "spamfssubmit"
    SPAMFSCHKSUM = "spamfschksum"
    SPAMFSURL = "spamfsurl"
    SPAMHELODNS = "spamhelodns"
    SPAMRADDRDNS = "spamraddrdns"
    SPAMRBL = "spamrbl"
    SPAMHDRCHECK = "spamhdrcheck"
    SPAMFSPHISH = "spamfsphish"


# ============================================================================
# Main Model
# ============================================================================

class ProfileModel(BaseModel):
    """
    Pydantic model for emailfilter/profile configuration.
    
    Configure Email Filter profiles.
    
    Validation Rules:        - name: max_length=47 pattern=        - comment: max_length=255 pattern=        - feature_set: pattern=        - replacemsg_group: max_length=35 pattern=        - spam_log: pattern=        - spam_log_fortiguard_response: pattern=        - spam_filtering: pattern=        - external: pattern=        - options: pattern=        - imap: pattern=        - pop3: pattern=        - smtp: pattern=        - mapi: pattern=        - msn_hotmail: pattern=        - yahoo_mail: pattern=        - gmail: pattern=        - other_webmails: pattern=        - spam_bword_threshold: min=0 max=2147483647 pattern=        - spam_bword_table: min=0 max=4294967295 pattern=        - spam_bal_table: min=0 max=4294967295 pattern=        - spam_mheader_table: min=0 max=4294967295 pattern=        - spam_rbl_table: min=0 max=4294967295 pattern=        - spam_iptrust_table: min=0 max=4294967295 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str = Field(max_length=47, description="Profile name.")    
    comment: str | None = Field(max_length=255, default=None, description="Comment.")    
    feature_set: Literal["flow", "proxy"] | None = Field(default="flow", description="Flow/proxy feature set.")    
    replacemsg_group: str | None = Field(max_length=35, default=None, description="Replacement message group.")  # datasource: ['system.replacemsg-group.name']    
    spam_log: Literal["disable", "enable"] | None = Field(default="enable", description="Enable/disable spam logging for email filtering.")    
    spam_log_fortiguard_response: Literal["disable", "enable"] | None = Field(default="disable", description="Enable/disable logging FortiGuard spam response.")    
    spam_filtering: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable spam filtering.")    
    external: Literal["enable", "disable"] | None = Field(default="disable", description="Enable/disable external Email inspection.")    
    options: list[ProfileOptionsEnum] = Field(default_factory=list, description="Options.")    
    imap: ProfileImap | None = Field(default=None, description="IMAP.")    
    pop3: ProfilePop3 | None = Field(default=None, description="POP3.")    
    smtp: ProfileSmtp | None = Field(default=None, description="SMTP.")    
    mapi: ProfileMapi | None = Field(default=None, description="MAPI.")    
    msn_hotmail: ProfileMsnHotmail | None = Field(default=None, description="MSN Hotmail.")    
    yahoo_mail: ProfileYahooMail | None = Field(default=None, description="Yahoo! Mail.")    
    gmail: ProfileGmail | None = Field(default=None, description="Gmail.")    
    other_webmails: ProfileOtherWebmails | None = Field(default=None, description="Other supported webmails.")    
    spam_bword_threshold: int | None = Field(ge=0, le=2147483647, default=10, description="Spam banned word threshold.")    
    spam_bword_table: int | None = Field(ge=0, le=4294967295, default=0, description="Anti-spam banned word table ID.")  # datasource: ['emailfilter.bword.id']    
    spam_bal_table: int | None = Field(ge=0, le=4294967295, default=0, description="Anti-spam block/allow list table ID.")  # datasource: ['emailfilter.block-allow-list.id']    
    spam_mheader_table: int | None = Field(ge=0, le=4294967295, default=0, description="Anti-spam MIME header table ID.")  # datasource: ['emailfilter.mheader.id']    
    spam_rbl_table: int | None = Field(ge=0, le=4294967295, default=0, description="Anti-spam DNSBL table ID.")  # datasource: ['emailfilter.dnsbl.id']    
    spam_iptrust_table: int | None = Field(ge=0, le=4294967295, default=0, description="Anti-spam IP trust table ID.")  # datasource: ['emailfilter.iptrust.id']    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    @field_validator('replacemsg_group')
    @classmethod
    def validate_replacemsg_group(cls, v: Any) -> Any:
        """
        Validate replacemsg_group field.
        
        Datasource: ['system.replacemsg-group.name']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('spam_bword_table')
    @classmethod
    def validate_spam_bword_table(cls, v: Any) -> Any:
        """
        Validate spam_bword_table field.
        
        Datasource: ['emailfilter.bword.id']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('spam_bal_table')
    @classmethod
    def validate_spam_bal_table(cls, v: Any) -> Any:
        """
        Validate spam_bal_table field.
        
        Datasource: ['emailfilter.block-allow-list.id']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('spam_mheader_table')
    @classmethod
    def validate_spam_mheader_table(cls, v: Any) -> Any:
        """
        Validate spam_mheader_table field.
        
        Datasource: ['emailfilter.mheader.id']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('spam_rbl_table')
    @classmethod
    def validate_spam_rbl_table(cls, v: Any) -> Any:
        """
        Validate spam_rbl_table field.
        
        Datasource: ['emailfilter.dnsbl.id']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    @field_validator('spam_iptrust_table')
    @classmethod
    def validate_spam_iptrust_table(cls, v: Any) -> Any:
        """
        Validate spam_iptrust_table field.
        
        Datasource: ['emailfilter.iptrust.id']
        
        Note:
            This validator only checks basic constraints.
            To validate that referenced object exists, query the API.
        """
        # Basic validation passed via Field() constraints
        # Additional datasource validation could be added here
        return v    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "ProfileModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)
    # ========================================================================
    # Datasource Validation Methods
    # ========================================================================    
    async def validate_replacemsg_group_references(self, client: Any) -> list[str]:
        """
        Validate replacemsg_group references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - system/replacemsg-group        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileModel(
            ...     replacemsg_group="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_replacemsg_group_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.emailfilter.profile.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "replacemsg_group", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.system.replacemsg_group.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Replacemsg-Group '{value}' not found in "
                "system/replacemsg-group"
            )        
        return errors    
    async def validate_spam_bword_table_references(self, client: Any) -> list[str]:
        """
        Validate spam_bword_table references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - emailfilter/bword        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileModel(
            ...     spam_bword_table="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_spam_bword_table_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.emailfilter.profile.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "spam_bword_table", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.emailfilter.bword.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Spam-Bword-Table '{value}' not found in "
                "emailfilter/bword"
            )        
        return errors    
    async def validate_spam_bal_table_references(self, client: Any) -> list[str]:
        """
        Validate spam_bal_table references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - emailfilter/block-allow-list        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileModel(
            ...     spam_bal_table="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_spam_bal_table_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.emailfilter.profile.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "spam_bal_table", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.emailfilter.block_allow_list.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Spam-Bal-Table '{value}' not found in "
                "emailfilter/block-allow-list"
            )        
        return errors    
    async def validate_spam_mheader_table_references(self, client: Any) -> list[str]:
        """
        Validate spam_mheader_table references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - emailfilter/mheader        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileModel(
            ...     spam_mheader_table="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_spam_mheader_table_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.emailfilter.profile.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "spam_mheader_table", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.emailfilter.mheader.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Spam-Mheader-Table '{value}' not found in "
                "emailfilter/mheader"
            )        
        return errors    
    async def validate_spam_rbl_table_references(self, client: Any) -> list[str]:
        """
        Validate spam_rbl_table references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - emailfilter/dnsbl        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileModel(
            ...     spam_rbl_table="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_spam_rbl_table_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.emailfilter.profile.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "spam_rbl_table", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.emailfilter.dnsbl.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Spam-Rbl-Table '{value}' not found in "
                "emailfilter/dnsbl"
            )        
        return errors    
    async def validate_spam_iptrust_table_references(self, client: Any) -> list[str]:
        """
        Validate spam_iptrust_table references exist in FortiGate.
        
        This method checks if referenced objects exist by calling exists() on
        the appropriate API endpoints. This is an OPTIONAL validation step that
        can be called before posting to the API to catch reference errors early.
        
        Datasource endpoints checked:
        - emailfilter/iptrust        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of validation error messages (empty if all valid)
            
        Example:
            >>> from hfortix_fortios import FortiOS
            >>> 
            >>> fgt = FortiOS(host="192.168.1.1", token="your-token")
            >>> policy = ProfileModel(
            ...     spam_iptrust_table="invalid-name",
            ... )
            >>> 
            >>> # Validate before posting
            >>> errors = await policy.validate_spam_iptrust_table_references(fgt._client)
            >>> if errors:
            ...     print("Validation failed:", errors)
            ... else:
            ...     result = await fgt.api.cmdb.emailfilter.profile.post(policy.to_fortios_dict())
        """
        errors: list[str] = []
        
        # Validate scalar field
        value = getattr(self, "spam_iptrust_table", None)
        if not value:
            return errors
        
        # Check all datasource endpoints
        found = False
        if await client.api.cmdb.emailfilter.iptrust.exists(value):
            found = True
        
        if not found:
            errors.append(
                f"Spam-Iptrust-Table '{value}' not found in "
                "emailfilter/iptrust"
            )        
        return errors    
    async def validate_all_references(self, client: Any) -> list[str]:
        """
        Validate ALL datasource references in this model.
        
        Convenience method that runs all validate_*_references() methods
        and aggregates the results.
        
        Args:
            client: FortiOS client instance (from fgt._client)
            
        Returns:
            List of all validation errors found
            
        Example:
            >>> errors = await policy.validate_all_references(fgt._client)
            >>> if errors:
            ...     for error in errors:
            ...         print(f"  - {error}")
        """
        all_errors = []
        
        errors = await self.validate_replacemsg_group_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_spam_bword_table_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_spam_bal_table_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_spam_mheader_table_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_spam_rbl_table_references(client)
        all_errors.extend(errors)        
        errors = await self.validate_spam_iptrust_table_references(client)
        all_errors.extend(errors)        
        return all_errors

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "ProfileModel",    "ProfileImap",    "ProfilePop3",    "ProfileSmtp",    "ProfileMapi",    "ProfileMsnHotmail",    "ProfileYahooMail",    "ProfileGmail",    "ProfileOtherWebmails",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.2
# Generated: 2026-01-21T16:24:00.624000Z
# ============================================================================


"""FortiOS CMDB - CustomLanguage category"""

from .create import Create
from .download import Download
from .update import Update

__all__ = [
    "Create",
    "CustomLanguage",
    "Download",
    "Update",
]


class CustomLanguage:
    """CustomLanguage endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """CustomLanguage endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.create = Create(client)
        self.download = Download(client)
        self.update = Update(client)
